// file: Convolution.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.net.*;
import java.io.*; 
import java.applet.*; 

/**
 * begins the applet and adds main GUI components to the applet
 *
 * hierarchy: JApplet->Convolution
 */
public class Convolution extends JApplet {
    
    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************

    // declare components to read the help files
    //
    URL url;
    Vector<URL> urlvec = new Vector<URL>(5, 1);

    // declare the dignal and impulse data objects
    //
    Data signal = new Data();
    Data impulse = new Data();
    
    // convolution size = N + M - 1
    //
    Data output = new Data(signal.getSize() + impulse.getSize());
    
    // declare gui components
    //
    MainMenu mainMenu;
    JPanel mainPanel = new JPanel();    
    OutputFrame magnitudePanel = new OutputFrame();
    SignalPanel signalPanel = new SignalPanel(signal, output);
    SystemPanel impulsePanel = new SystemPanel(impulse, output);
    OutputPanel outputPanel = new OutputPanel(signal, impulse, output,
					       signalPanel, impulsePanel);
    ControlPanel controlPanel = new ControlPanel(outputPanel, signalPanel,
						  impulsePanel,
						  magnitudePanel);

    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************

    /**
     * Default class constructor.  Includes a
     * hack to avoid ugly message about system event access check.
     */
    public Convolution() {
        getRootPane().putClientProperty("defeatSystemEventQueueCheck",
                                            Boolean.TRUE);
    }

    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************

    /**
     * routine called when applet Convolution begins sets layout of the 
     * applet and adds an input and output panel to the applet
     */
    public void init() {
	
	super.init();
	
	// declare components to read the help files
	//
	try {
	    url = new URL(getCodeBase() + "doc/help_release.txt");
	    urlvec.addElement(url);
	    url = new URL(getCodeBase() + "doc/help_info_support.txt");
	    urlvec.addElement(url);
	    url = new URL(getCodeBase() + "doc/help_overview.txt");
	    urlvec.addElement(url);
	    url = new URL(getCodeBase() + "doc/tutorial.html");
	    urlvec.addElement(url);
	    url = new URL(getCodeBase() + "doc/help_plugins.txt");
	    urlvec.addElement(url);
	}
	catch(MalformedURLException me) { }
	
	// get the applet context
	//
	AppletContext context = getAppletContext(); 

	// define the main menu component
	//
	mainMenu = new MainMenu(signalPanel, impulsePanel,
				outputPanel, controlPanel,
				magnitudePanel, urlvec, context);

	// define the content pane of the toplevel container
	//
	JPanel contentPane = new JPanel();
	
	// specify layout manager as GridBag
	//
	GridBagLayout gridbag = new GridBagLayout();
	
	contentPane.setLayout(null);
	addNotify();
	
	// add main panel to applet
	//
	mainPanel.setLayout(gridbag);
	
	// set the applet dimension and background color
	//
	mainPanel.setSize(new Dimension(660, 550));
	mainPanel.setBackground(Color.lightGray);
	contentPane.add(mainPanel);
	
	// add the input panel across the top
	//
	mainMenu.constrain(mainPanel, mainMenu, 0, 0, 
                            GridBagConstraints.REMAINDER, 1, 
                            GridBagConstraints.BOTH, 
                            GridBagConstraints.NORTHWEST, 
                            1, 0.1, 2, 2, 2, 2);

	controlPanel.constrain(mainPanel, controlPanel, 0, 1, 
				1, 1, 
				GridBagConstraints.BOTH,
				GridBagConstraints.NORTHWEST, 
				0.61, 0.2, 2, 2, 2, 2);
	
	signalPanel.constrain(mainPanel, signalPanel, 1, 1, 
			       1, 1, 
			       GridBagConstraints.BOTH, 
			       GridBagConstraints.NORTHWEST, 
			       1, 0.2, 2, 2, 2, 2);
	
	impulsePanel.constrain(mainPanel, impulsePanel, 2, 1, 
				GridBagConstraints.REMAINDER, 1, 
				GridBagConstraints.BOTH, 
				GridBagConstraints.NORTHWEST, 
				1, 0.2, 2, 2, 2, 2);
	
	// add the output panel across the bottom
	//
	outputPanel.constrain(mainPanel, outputPanel, 0, 2, 
			       GridBagConstraints.REMAINDER, 1, 
			       GridBagConstraints.BOTH,
			       GridBagConstraints.NORTHWEST, 
			       1, 1, 2, 2, 2, 2);
	
	// add components to each panel
	//
	mainMenu.addComponents();
	controlPanel.addComponents();
	signalPanel.addComponents();
	impulsePanel.addComponents();
	outputPanel.addComponents();
	
	setContentPane(contentPane);
    }
}

// 
// end of file

