// file: SignalPanel.java
//

// import necessary java libraries
//
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * Creates a basic canvas for which a drawing canvas, a control bar
 * for the signal paramaters, and a control bar for drawing are added
 *
 * hierarchy: JPanel->SubPanel->SignalPanel
 */
public class SignalPanel extends SubPanel {

    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************

    /**
     * place for data to be written to once read from signal
     * conatins flags that are set and read depending on apllication
     */
    Data data;
    
    /**
     * a set of data that may be loaded from an external widget
     */
    Data copyData;
    
    /**
     * define the signal panel title
     */
    String signalTitle = new String("Signal Input:");

    /**
     * components
     */
    SignalDrawingCanvas wavePanel;

    // declare the default fonts
    //
    Font currentFont = getFont();
    Font newFont = new Font(currentFont.getName(), currentFont.getStyle(), 12);

    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************
    
    /**
     * creates a a basic frame work to add plotting gui components
     *
     * @param d input signal data samples
     * @param o convolved waveform data samples
     */
    SignalPanel(Data d, Data o)  {

	super();
	
	data = d;
	copyData = o;
	
	// pass the data into all parts of the signal panel for manipulation
	//
	wavePanel = new SignalDrawingCanvas(data);
    }

    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************

    /**
     * adds a control panel, drawing controls and a canvas to the signal panel
     */ 
    public void addComponents() {

	// set a titled border for the canvas
	//
	Border titleBorder = BorderFactory.createTitledBorder(
			     BorderFactory.createEmptyBorder(), 
			     signalTitle, 
			     TitledBorder.LEFT, 
			     TitledBorder.ABOVE_TOP, 
			     newFont, 
			     Color.black);
	this.setBorder(titleBorder);
	
	// place signal drawing panel on top of signal panel
	//
	constrain(this, wavePanel, 0, 0, 
		  GridBagConstraints.REMAINDER, 1, 
		  GridBagConstraints.BOTH,
		  GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 2, 2, 2);
    }
}

//
// end of file







