// file: SystemPanel.java
//

// import necessary java libraries
//
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * Creates a basic canvas for which a drawing canvas, a control bar
 * for the input paramaters, and a control bar for drawing are added
 *
 * hierarchy: JPanel->SubPanel->SystemPanel
 */
public class SystemPanel extends SubPanel {

    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************

    /**
     * place for data to be written to once read from input
     * conatins flags that are set and read depending on apllication
     */
    Data data;
    
    /**
     * a set of data that may be loaded from an external widget
     */
    Data copyData;
    
    /**
     * define the system panel title
     */
    String systemTitle = new String("System Response:");

    /**
     * components
     */
    SystemDrawingCanvas wavePanel;

    // declare the default fonts
    //
    Font currentFont = getFont();
    Font newFont = new Font(currentFont.getName(), currentFont.getStyle(), 12);

    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************
  
    /**
     * constructor for class takes the impulse and output data samples
     * as well as the output panel 
     *
     * @param d input impulse data samples
     * @param o convolved waveform data samples
     */
    public SystemPanel(Data d, Data o)  {

	super();

	data = d;
	copyData = o;
	
	// pass the data into all parts of the input panel for manipulation
	//
	wavePanel = new SystemDrawingCanvas(data);
    }
    
    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************

    /**
     * add components to the system input panel
     */
    public void addComponents() {

	// set a titled border for the canvas
	//
	Border titleBorder = BorderFactory.createTitledBorder(
			     BorderFactory.createEmptyBorder(), 
			     systemTitle, 
			     TitledBorder.LEFT,
			     TitledBorder.ABOVE_TOP, 
			     newFont, 
			     Color.black);
	this.setBorder(titleBorder);
	
	// place input drawing panel on top of input panel
	//
	constrain(this, wavePanel, 1, 1, 
		  GridBagConstraints.REMAINDER, 1, 
		  GridBagConstraints.BOTH,
		  GridBagConstraints.NORTHEAST, 1.0, 1.0, 2, 2, 2, 2);
    }
}

//
// end of file








